Documentation
¶
Index ¶
- Variables
- func Asset(name string) ([]byte, error)
- func AssetDigest(name string) ([sha256.Size]byte, error)
- func AssetDir(name string) ([]string, error)
- func AssetInfo(name string) (os.FileInfo, error)
- func AssetNames() []string
- func AssetString(name string) (string, error)
- func Digests() (map[string][sha256.Size]byte, error)
- func MustAsset(name string) []byte
- func MustAssetString(name string) string
- func RestoreAsset(dir, name string) error
- func RestoreAssets(dir, name string) error
- type ChainMessage
- type ChartEntries
- type ChartEntry
- type Config
- type Dashboard
- type GeneralMessage
- type HomeMessage
- type LogsMessage
- type Message
- type NetworkMessage
- type SystemMessage
- type TxPoolMessage
Constants ¶
This section is empty.
Variables ¶
var DefaultConfig = Config{ Host: "localhost", Port: 8080, Refresh: 5 * time.Second, }
DefaultConfig包含仪表板的默认设置。 DefaultConfig contains default settings for the dashboard.
Functions ¶
func Asset ¶
Asset loads and returns the asset for the given name. It returns an error if the asset could not be found or could not be loaded.
func AssetDigest ¶
AssetDigest returns the digest of the file with the given name. It returns an error if the asset could not be found or the digest could not be loaded.
func AssetDir ¶
AssetDir returns the file names below a certain directory embedded in the file by go-bindata. For example if you run go-bindata on data/... and data contains the following hierarchy:
data/
foo.txt
img/
a.png
b.png
then AssetDir("data") would return []string{"foo.txt", "img"}, AssetDir("data/img") would return []string{"a.png", "b.png"}, AssetDir("foo.txt") and AssetDir("notexist") would return an error, and AssetDir("") will return []string{"data"}.
func AssetInfo ¶
AssetInfo loads and returns the asset info for the given name. It returns an error if the asset could not be found or could not be loaded.
func AssetString ¶
AssetString returns the asset contents as a string (instead of a []byte).
func MustAsset ¶
MustAsset is like Asset but panics when Asset would return an error. It simplifies safe initialization of global variables.
func MustAssetString ¶
MustAssetString is like AssetString but panics when Asset would return an error. It simplifies safe initialization of global variables.
func RestoreAsset ¶
RestoreAsset restores an asset under the given directory.
func RestoreAssets ¶
RestoreAssets restores an asset under the given directory recursively.
Types ¶
type ChainMessage ¶
type ChainMessage struct {
}
type ChartEntries ¶
type ChartEntries []*ChartEntry
type ChartEntry ¶
type Config ¶
type Config struct {
//host是启动仪表板服务器的主机接口。如果此/字段为空,则不会启动仪表板。
// Host is the host interface on which to start the dashboard server. If this
// field is empty, no dashboard will be started.
Host string `toml:",omitempty"`
//Port是启动仪表板服务器的TCP端口号。/默认值为/有效,将随机选择一个端口号(有用/用于临时节点)。
// Port is the TCP port number on which to start the dashboard server. The
// default zero value is/ valid and will pick a port number randomly (useful
// for ephemeral nodes).
Port int `toml:",omitempty"`
//Refresh是数据更新的刷新速率,将经常收集chartEntry。
// Refresh is the refresh rate of the data updates, the chartEntry will be collected this often.
Refresh time.Duration `toml:",omitempty"`
}
Config contains the configuration parameters of the dashboard./Config包含仪表板的配置参数。
type Dashboard ¶
type Dashboard struct {
// contains filtered or unexported fields
}
Dashboard contains the dashboard internals.
type GeneralMessage ¶
type HomeMessage ¶
type HomeMessage struct {
}
type LogsMessage ¶
type LogsMessage struct {
Log []string `json:"log,omitempty"`
}
type Message ¶
type Message struct {
General *GeneralMessage `json:"general,omitempty"`
Home *HomeMessage `json:"home,omitempty"`
Chain *ChainMessage `json:"chain,omitempty"`
TxPool *TxPoolMessage `json:"txpool,omitempty"`
Network *NetworkMessage `json:"network,omitempty"`
System *SystemMessage `json:"system,omitempty"`
Logs *LogsMessage `json:"logs,omitempty"`
}
type NetworkMessage ¶
type NetworkMessage struct {
}
type SystemMessage ¶
type SystemMessage struct {
ActiveMemory ChartEntries `json:"activeMemory,omitempty"`
VirtualMemory ChartEntries `json:"virtualMemory,omitempty"`
NetworkIngress ChartEntries `json:"networkIngress,omitempty"`
NetworkEgress ChartEntries `json:"networkEgress,omitempty"`
ProcessCPU ChartEntries `json:"processCPU,omitempty"`
SystemCPU ChartEntries `json:"systemCPU,omitempty"`
DiskRead ChartEntries `json:"diskRead,omitempty"`
DiskWrite ChartEntries `json:"diskWrite,omitempty"`
}
type TxPoolMessage ¶
type TxPoolMessage struct {
}